@@ -347,6 +347,8 @@ typedef struct
 	char	  **colnames;
 	ExtStr	   *lines;
 	char		linestyle;
+
+	int			nlines;			/* for debug purposes */
 } ExportState;
 
 /*
@@ -410,6 +412,8 @@ process_item(ExportState *expstate,
 
 		if (typ == 'N' && !is_colname && !has_continue_mark)
 		{
+			expstate->nlines += 1;
+
 			if (expstate->format == CLIPBOARD_FORMAT_INSERT)
 				fputs(");\n", expstate->fp);
 			else
@@ -577,7 +581,11 @@ process_item(ExportState *expstate,
 			fwrite(field, size, 1, expstate->fp);
 		}
 		else
+		{
 			fputc('\n', expstate->fp);
+			expstate->nlines += 1;
+		}
+
 	}
 
 	else if (expstate->format == CLIPBOARD_FORMAT_PIPE_SEPARATED)
@@ -606,7 +614,10 @@ process_item(ExportState *expstate,
 				}
 			}
 			else
+			{
 				fputc('\n', expstate->fp);
+				expstate->nlines += 1;
+			}
 		}
 	}
 
@@ -624,6 +635,8 @@ process_item(ExportState *expstate,
 		{
 			errno = 0;
 			fputc('\n', expstate->fp);
+			expstate->nlines += 1;
+
 		}
 		else
 		{
@@ -746,6 +759,9 @@ export_data(Options *opts,
 	bool	save_column_names = false;
 	bool	has_selection;
 
+	int		debug_read_rows = 0;
+	int		debug_processed_rows = 0;
+
 	int		min_row = desc->first_data_row;
 	int		max_row = desc->last_row;
 
@@ -768,6 +784,7 @@ export_data(Options *opts,
 	expstate.columns = desc->columns;
 	expstate.copy_line_extended = (cmd == cmd_CopyLineExtended);
 	expstate.linestyle = desc->linestyle;
+	expstate.nlines = 0;
 
 	current_state->errstr = NULL;
 
@@ -944,6 +961,10 @@ export_data(Options *opts,
 		}
 	}
 
+	log_row("export: desc->first_data_row: %d, desc->last_data_row: %d",
+			desc->first_data_row, desc->last_data_row);
+	log_row("export: min_row: %d, max_row: %d", min_row, max_row);
+
 	init_lbi_ddesc(&lbi, desc, 0);
 
 	while (lbi_set_mark_next(&lbi, &lbm))
@@ -961,6 +982,8 @@ export_data(Options *opts,
 
 		(void) lbm_get_line(&lbm, &rowstr, &linfo, &rn);
 
+		debug_read_rows += 1;
+
 		/* reduce rows from export */
 		if (rn >= desc->first_data_row && rn <= desc->last_data_row)
 		{
@@ -1013,6 +1036,8 @@ export_data(Options *opts,
 		if (format != CLIPBOARD_FORMAT_TEXT)
 			continuation_mark = linfo && linfo->mask & LINEINFO_CONTINUATION;
 
+		debug_processed_rows += 1;
+
 		/*
 		 * line parser - separates fields on line
 		 */
@@ -1074,6 +1099,8 @@ export_data(Options *opts,
 
 exit_export:
 
+	log_row("export: read rows: %d, procesed rows: %d", debug_read_rows, debug_processed_rows);
+
 	if (expstate.colnames)
 	{
 		int		i;
@@ -1097,5 +1124,7 @@ exit_export:
 		free(expstate.lines);
 	}
 
+	log_row("exported %d rows with result %d", expstate.nlines, isok);
+
 	return isok;
 }
