@@ -44,13 +44,14 @@ static char	pathname[MAXPATHLEN] = "";
 FILE   *f_tty = NULL;						/* ncurses input stream */
 FILE   *f_data = NULL;						/* content input */
 
-int		f_data_opts = 0;						/* describe properties of content input */
+unsigned int f_data_opts = 0;						/* describe properties of content input */
 
 int		f_data_fileno = -1;					/* content input used by poll */
 
 static struct pollfd fds[2];
 
 static long last_data_pos = -1;
+static int open_data_stream_prev_errno = 0;
 
 #if defined(HAVE_INOTIFY) || defined(HAVE_KQUEUE)
 
@@ -68,7 +69,6 @@ static NCursesEventData saved_event;
 static bool saved_event_is_valid = false;
 
 static bool close_f_tty = false;
-static bool close_f_data = false;
 
 #ifdef PDCURSES
 
@@ -430,6 +430,7 @@ _get_pspg_event(NCursesEventData *nced,
 	if (!only_tty_events)
 	{
 		if (current_state->stream_mode &&
+			(f_data_opts & STREAM_IS_OPEN) &&
 			!(f_data_opts & STREAM_IS_FILE) &&
 			!(f_data_opts & STREAM_IS_CLOSED))
 		{
@@ -871,16 +872,21 @@ open_data_stream(Options *opts)
 		 * in open time. But it doesn't look like robust solution.
 		 */
 		f_data = fopen(locpathname, mode);
-		f_data_opts = 0;
 		if (!f_data)
 		{
 			/* save errno, and prepare error message */
 			current_state->_errno = errno;
+
+			/*
+			 * In watch mode, the file can be created in next cycle. In this case
+			 * we don't want skip current content, when stream mode is active.
+			 */
+			open_data_stream_prev_errno = errno;
 			format_error("cannot to open file \"%s\" (%s)", pathname, strerror(errno));
 			return false;
 		}
 
-		close_f_data = true;
+		f_data_opts = STREAM_IS_OPEN | STREAM_CAN_BE_CLOSED;
 	}
 	else
 	{
@@ -888,10 +894,10 @@ open_data_stream(Options *opts)
 		pathname[0] = '\0';
 
 		/* use stdin as input if query cannot be used as source */
-		if (!opts->query)
+		if (!opts->query && !opts->querystream)
 		{
 			f_data = stdin;
-			f_data_opts = STREAM_IS_PIPE;
+			f_data_opts = STREAM_IS_OPEN | STREAM_IS_PIPE;
 		}
 	}
 
@@ -929,8 +935,15 @@ open_data_stream(Options *opts)
 
 #endif
 
-				fseek(f_data, 0L, SEEK_END);
+				/*
+				 * In stream mode skip current content if this file
+				 * is opened first time.
+				 */
+				fseek(f_data, 0L,
+					  open_data_stream_prev_errno == ENOENT ? SEEK_SET : SEEK_END);
+
 				last_data_pos = ftell(f_data);
+				open_data_stream_prev_errno = 0;
 			}
 			else
 			{
@@ -1070,9 +1083,7 @@ open_data_stream(Options *opts)
 void
 close_data_stream(void)
 {
-//leave("close file");
-
-	if (close_f_data)
+	if (f_data_opts & STREAM_CAN_BE_CLOSED & STREAM_IS_OPEN)
 	{
 		fclose(f_data);
 
@@ -1088,7 +1099,8 @@ close_data_stream(void)
 
 #endif
 
-		close_f_data = false;
+		f_data_opts &= ~(STREAM_CAN_BE_CLOSED);
+		f_data_opts &= ~(STREAM_IS_OPEN);
 	}
 
 	/*
@@ -1192,6 +1204,11 @@ detect_file_truncation(void)
 	{
 		struct stat stats;
 
+		/*
+		 * This detection based on size is weak. It doesn't detect
+		 * case when old content is overwritten by new content with
+		 * same or bigger size.
+		 */
 		if (fstat(fileno(f_data), &stats) == 0)
 		{
 			if (stats.st_size < last_data_pos)
