@@ -705,6 +705,7 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 		desc->border_bottom_row = -1;
 		desc->first_data_row = -1;
 		desc->last_data_row = -1;
+		desc->fallback_last_data_row = false;
 		desc->is_expanded_mode = false;
 		desc->headline_transl = NULL;
 		desc->cranges = NULL;
@@ -903,6 +904,7 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 		{
 			desc->border_bottom_row = nrows;
 			desc->last_data_row = nrows - 1;
+			desc->fallback_last_data_row = false;
 			desc->load_data_rows = false;
 			log_row("next row will be desc row");
 
@@ -916,6 +918,7 @@ readfile(Options *opts, DataDesc *desc, StateData *state)
 			/* Outer border is repeated in expanded mode, use last detected row */
 			desc->border_bottom_row = nrows;
 			desc->last_data_row = nrows - 1;
+			desc->fallback_last_data_row = false;
 			log_row("next row will be desc row");
 		}
 
@@ -978,6 +981,9 @@ next_row:
 
 	log_row("read rows %d", nrows);
 
+	if (opts->querystream)
+		goto final;
+
 	if (initial_run)
 	{
 		/*
@@ -1008,8 +1014,13 @@ next_row:
 		 * fallback, but can be fixed later, when border_type
 		 * will be known.
 		 */
-		if (desc->last_data_row == -1)
+		if (desc->last_data_row == -1 ||
+			(desc->last_data_row != -1 && desc->fallback_last_data_row))
+		{
 			desc->last_data_row = desc->last_row - 1;
+			desc->fallback_last_data_row = true;
+			log_row("set fallback_last_data_row to %d", desc->last_data_row);
+		}
 
 		if (desc->border_head_row >= 1)
 			desc->namesline = desc->rows.rows[desc->border_head_row - 1];
@@ -1037,10 +1048,13 @@ broken_format:
 
 		/* there are not a data set */
 		desc->last_data_row = desc->last_row;
+		desc->fallback_last_data_row = false;
 		desc->title_rows = 0;
 		desc->title[0] = '\0';
 	}
 
+final:
+
 #ifdef DEBUG_PIPE
 
 	print_duration(start_sec, start_ms, "read file");
@@ -1055,7 +1069,6 @@ broken_format:
 
 #endif
 
-
 	return true;
 }
 
@@ -1341,8 +1354,7 @@ translate_headline(DataDesc *desc)
 		if (desc->border_type == 0 && last_black_char)
 		{
 			last_black_char[0] = 'd';
-			last_black_char[1] = 'R';
-			last_black_char[2] = '\0';
+			last_black_char[1] = '\0';
 		}
 
 		/* trim ending spaces */
@@ -1613,6 +1625,8 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 	bool		only_digits = false;
 	bool		only_digits_with_point = false;
 	bool		skip_initial_spaces = true;
+	bool		found_plus_sign = false;
+	bool		found_minus_sign = false;
 	int			x = 0;
 	long		mp = 1;
 
@@ -1639,6 +1653,18 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 						str += 1;
 						continue;
 					}
+					else if ((c == '-' || c == '+') &&
+							  !(found_plus_sign || found_minus_sign))
+					{
+						if (c == '-')
+							found_minus_sign = true;
+						else
+							found_plus_sign = true;
+
+						x += 1;
+						str += 1;
+						continue;
+					}
 
 					/* first char should be a digit */
 					if (!isdigit(c))
@@ -1762,7 +1788,7 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 				mp = 1024l * 1024 * 1024 * 1024;
 
 			/* units used by nushell */
-			if (nstreq(first_nospace_nodigit, "B"))
+			else if (nstreq(first_nospace_nodigit, "B"))
 				mp = 1l;
 			else if (nstreq(first_nospace_nodigit, "KiB"))
 				mp = 1024l;
@@ -1775,7 +1801,10 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 
 				/* unknown unit */
 			else
+			{
+				log_row("unknown unit \"%s\"", first_nospace_nodigit);
 				return false;
+			}
 
 			*first_nospace_nodigit = '\0';
 		}
@@ -1797,6 +1826,10 @@ cut_numeric_value(char *str, int xmin, int xmax, double *d, bool border0, bool *
 		if (errno == 0)
 		{
 			*d = *d * mp;
+
+			if (found_minus_sign)
+				*d *= -1;
+
 			return true;
 		}
 	}
@@ -1849,8 +1882,11 @@ multilines_detection(DataDesc *desc)
 		{
 			/*
 			 * This implementation doesn't support old-ascii format
+			 *
+			 * Note - processed string can be shorter than headline for last
+			 * column with pset border 1
 			 */
-			while (pos < desc->headline_char_size)
+			while (pos < desc->headline_char_size && *str != '\0')
 			{
 				if (border0)
 				{
@@ -2047,9 +2083,15 @@ sort_by_string:
 		leave("unexpected processed rows after sort prepare");
 
 	if (detect_string_column)
+	{
+		log_row("string sort");
 		sort_column_text(sortbuf, sortbuf_pos, desc_sort);
+	}
 	else
+	{
+		log_row("numeric sort");
 		sort_column_num(sortbuf, sortbuf_pos, desc_sort);
+	}
 
 	lineno = desc->first_data_row;
 
